# -*- coding: utf-8 -*-

import json

from vaporize.core import get_url, handle_request, query
import vaporize.flavors
import vaporize.images
import vaporize.ipgroups
from vaporize.utils import DotDict

BACKUP_WEEKLY_DISABLED  = 'DISABLED'
BACKUP_WEEKLY_SUNDAY    = 'SUNDAY'
BACKUP_WEEKLY_MONDAY    = 'MONDAY'
BACKUP_WEEKLY_TUESDAY   = 'TUESDAY'
BACKUP_WEEKLY_WEDNESDAY = 'WEDNESDAY'
BACKUP_WEEKLY_THURSDAY  = 'THURSDAY'
BACKUP_WEEKLY_FRIDAY    = 'FRIDAY'
BACKUP_WEEKLY_SATURDAY  = 'SATURDAY'

BACKUP_DAILY_DISABLED    = 'DISABLED'
BACKUP_DAILY_H_0000_0200 = 'H_0000_0200'
BACKUP_DAILY_H_0200_0400 = 'H_0200_0400'
BACKUP_DAILY_H_0400_0600 = 'H_0400_0600'
BACKUP_DAILY_H_0600_0800 = 'H_0600_0800'
BACKUP_DAILY_H_0800_1000 = 'H_0800_1000'
BACKUP_DAILY_H_1000_1200 = 'H_1000_1200'
BACKUP_DAILY_H_1200_1400 = 'H_1200_1400'
BACKUP_DAILY_H_1400_1600 = 'H_1400_1600'
BACKUP_DAILY_H_1600_1800 = 'H_1600_1800'
BACKUP_DAILY_H_1800_2000 = 'H_1800_2000'
BACKUP_DAILY_H_2000_2200 = 'H_2000_2200'
BACKUP_DAILY_H_2200_0000 = 'H_2200_0000'


class BackupSchedule(DotDict):
    """A CloudServers Backup Schedule."""
    def __repr__(self):
        if 'daily' in self:
            return '<BackupSchedule %s>' % self['daily']
        if 'weekly' in self:
            return '<BackupSchedule %s>' % self['weekly']
        return super(BackupSchedule, self).__repr__()


class IP(DotDict):
    """A CloudServers IP Address."""
    def __repr__(self):
        if 'public' in self:
            return '<IP %s>' % self['public'][0]
        if 'private' in self:
            return '<IP %s>' % self['private'][0]
        return super(IP, self).__repr__()


class Server(DotDict):
    """A CloudServers Server."""
    def __repr__(self):
        if 'name' in self:
            return "<Server %s>" % self['name']
        return super(Server, self).__repr__()

    def __setitem__(self, key, value):
        if key == 'addresses':
            value = IP(value)
        super(Server, self).__setitem__(key, value)

    def reload(self):
        """Reload this Server (an implicit :func:`get`).

        :returns: An updated CloudServers Server.
        :rtype: :class:`Server`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        response = get(self['id'])
        self.update(response)
        return self

    def modify(self, name=None, password=None):
        """Modify this Server's name or root password.

        :param name: Change the Server's name
        :type name: str
        :param password: Change the Server's root password
        :type password: str
        :returns: A modified CloudServers Server.
        :rtype: :class:`Server`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = {'server': {}}
        if name is not None:
            data['server']['name'] = name
        if password is not None:
            data['server']['adminPass'] = password
        data = json.dumps(data)
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id'])])
        response = handle_request('put', url, data=data)
        if response:
            if name is not None:
                self['name'] = name
        return self

    def delete(self):
        """Delete this Server.

        .. warning::

            There is no confirmation step for this operation. When you delete a
            server it is permanent. If in doubt, create a backup image
            (:func:`vaporize.images.create`) first before deleting.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id'])])
        handle_request('delete', url)

    def ips(self):
        """
        Returns a list of public and private IPs for this Server.

        :returns: A list of public and private IPs for this Server.
        :rtype: A list of :class:`IP`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'ips'])
        response = handle_request('get', url, wrapper=IP,
                                  container='addresses')
        self['addresses'] = response
        return self['addresses']

    @property
    def public_ips(self):
        """Returns the Server's Public IP.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'ips', 'public'])
        response = handle_request('get', url, wrapper=IP)
        self['addresses'].update(response)
        return self['addresses']['public']

    @property
    def private_ips(self):
        """Returns the Server's Private IP.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'ips', 'private'])
        response = handle_request('get', url, wrapper=IP)
        self['addresses'].update(response)
        return self['addresses']['private']

    def share_ip(self, address, ipgroup, configure=True):
        """Share this Server's IP in a Shared IP Group.

        :param address: IP to share in the Shared IP Group
        :type address: str
        :param ipgroup: A :class:`vaporize.ipgroups.SharedIPGroup` or ``id``
        :type ipgroup: int or :class:`vaporize.ipgroups.SharedIPGroup`
        :param configure: Configure the shared IP on the Server
        :type configure: bool
        :returns: The Shared IP Group associated with this Server.
        :rtype: :class:`vaporize.ipgroups.SharedIPGroup`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        if isinstance(ipgroup, vaporize.ipgroups.SharedIPGroup):
            ipgroup = ipgroup.id
        ipgroup = int(ipgroup)
        data = json.dumps({'shareIp': {'sharedIpGroup': ipgroup,
                                       'configureServer': configure}})
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'ips', 'public', address])
        handle_request('put', url, data=data)

    def unshare_ip(self, address):
        """Unshare this Server's IP

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'ips', 'public', address])
        handle_request('delete', url)

    def soft_reboot(self):
        """Perform a soft reboot on this Server

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'reboot': {'type': 'SOFT'}})
        url = '/'.join([get_url('cloudservers'), 'servers',
                        str(self['id']), 'action'])
        handle_request('post', url, data)

    def hard_reboot(self):
        """Perform a hard reboot on this Server

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'reboot': {'type': 'HARD'}})
        url = '/'.join([get_url('cloudservers'), 'servers',
                        str(self['id']), 'action'])
        handle_request('post', url, data)

    def rebuild(self, image):
        """Rebuild this Server using a specified Image

        :param image: The Image or ``id``
        :type image: int or :class:`vaporize.images.Image`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        if isinstance(image, vaporize.images.Image):
            image = image.id
        image = int(image)
        data = json.dumps({'rebuild': {'imageId': int(image)}})
        url = '/'.join([get_url('cloudservers'), 'servers',
                        str(self['id']), 'action'])
        handle_request('post', url, data)

    def resize(self, flavor):
        """Resize this Server to a specific Flavor size

        :param flavor: The Flavor or ``id``
        :type flavor: int or :class:`vaporize.flavors.Flavor`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        if isinstance(flavor, vaporize.flavors.Flavor):
            flavor = flavor.id
        flavor = int(flavor)
        data = json.dumps({'resize': {'flavorId': flavor}})
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'action'])
        handle_request('post', url, data)

    def confirm_resize(self):
        """Confirm a successful resize operation

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'confirmResize': None})
        url = '/'.join([get_url('cloudservers'), 'servers',
                        str(self['id']), 'action'])
        handle_request('post', url, data)

    def revert_resize(self):
        """Revert an unsuccessful resize operation

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'revertResize': None})
        url = '/'.join([get_url('cloudservers'), 'servers',
                        str(self['id']), 'action'])
        handle_request('post', url, data)

    def backup_schedule(self):
        """Return this Server's backup schedule

        :return: :class:`BackupSchedule`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'backup_schedule'])
        return handle_request('get', url, wrapper=BackupSchedule,
                              container='backupSchedule')

    def enable_backup_schedule(self, weekly=None, daily=None):
        """Enable a backup schedule for this Server

        :param weekly: The weekly backup schedule
        :type weekly: str
        :param daily: The daily backup schedule
        :type daily: str

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = {'backupSchedule': {'enable': True}}
        if weekly is not None:
            data['backupSchedule']['weekly'] = weekly
        if daily is not None:
            data['backupSchedule']['daily'] = daily
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'backup_schedule'])
        handle_request('post', url, data)

    def disable_backup_schedule(self):
        """Disable a backup schedule for this Server

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'servers', str(self['id']),
                        'backup_schedule'])
        handle_request('delete', url)


def list(limit=None, offset=None, detail=False):
    """
    List of CloudServer Servers

    :param limit: Limit the result set to a certain number
    :type limit: int
    :param offset: Offset the result set by a certain number
    :type offset: int
    :param detail: Return detailed information about each Server
    :type detail: bool
    :returns: A list of CloudServers Servers.
    :rtype: List of :class:`Server`

    .. versionadded:: 0.1
    """
    url = [get_url('cloudservers'), 'servers']
    if detail:
        url.append('detail')
    url = '/'.join(url)
    if limit is not None or offset is not None:
        url = query(url, limit=limit, offset=offset)
    return handle_request('get', url, wrapper=Server, container='servers')


def get(id):
    """Return a Server using an ID

    :param id: The ``id`` of the Server to be retrieved
    :type id: int
    :return: A :class:`Server`

    .. versionadded:: 0.1
    """
    url = '/'.join([get_url('cloudservers'), 'servers', str(id)])
    return handle_request('get', url, wrapper=Server, container='server')


def create(name, image, flavor, metadata=None, files=None):
    """Create a CloudServers Server

    :param name: A Server's name
    :type name: str
    :param image: An Image or ``id``
    :type image: int or :class:`vaporize.images.Image`
    :param flavor: A Flavor or ``id``
    :type flavor: int or :class:`vaporize.flavors.Flavor`
    :param metadata: Optional meta data to include with Server
    :type metadata: dict
    :param files: A list of files to load on Server
    :type files: dict
    :returns: A shiny new CloudServers Server.
    :rtype: :class:`Server`

    .. versionadded:: 0.1
    """
    if isinstance(image, vaporize.images.Image):
        image = image.id
    image = int(image)
    if isinstance(flavor, vaporize.flavors.Flavor):
        flavor = flavor.id
    flavor = int(flavor)
    data = {'server': {'name': name,
                       'imageId': image,
                       'flavorId': flavor,
                       'metadata': metadata or {},
                       'personality': []}}
    if isinstance(files, dict):
        for path, contents in list(files.items()):
            data['personality'].append({'path': path, 'contents': contents})
    data = json.dumps(data)
    url = '/'.join([get_url('cloudservers'), 'servers'])
    return handle_request('post', url, data, Server, 'server')



# -*- coding: utf-8 -*-

import json

from vaporize.core import get_url, handle_request, query
import vaporize.servers
from vaporize.utils import DotDict


class SharedIPGroup(DotDict):
    """A Cloudservers Shared IP Group."""
    def __repr__(self):
        if 'name' in self:
            return '<SharedIPGroup %s>' % self['name']
        return super(SharedIPGroup, self).__repr__()

    def __setitem__(self, key, value):
        if key == 'sharedIpGroupId':
            key = 'id'
        elif key == 'configuredServer':
            key = 'configured'
        super(SharedIPGroup, self).__setitem__(key, value)

    def delete(self):
        """Delete this Shared IP Group.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'shared_ip_groups',
                        str(self['id'])])
        handle_request('delete', url)


def list(limit=None, offset=None, detail=False):
    """Returns a list of Shared IP Groups.

    :param limit: Limit the result set by a certain number
    :type limit: int
    :param offset: Offset the result set by a certain number
    :type offset: int
    :param detail: Return additional details about each Shared IP Group
    :type detail: bool
    :returns: A list of Shared IP Groups.
    :rtype: A list of :class:`SharedIPGroup`

    .. versionadded:: 0.1
    """
    url = [get_url('cloudservers'), 'shared_ip_groups']
    if detail:
        url.append('detail')
    url = '/'.join(url)
    if limit is not None or offset is not None:
        url = query(url, limit=limit, offset=offset)
    return handle_request('get', url, wrapper=SharedIPGroup,
                          container='sharedIpGroups')


def get(id):
    """Return a Shared IP Group by ID.

    :param id: The ID of the Shared IP Group to retrieve
    :type id: int
    :returns: A :class:`SharedIPGroup`

    .. versionadded:: 0.1
    """
    url = '/'.join([get_url('cloudservers'), 'shared_ip_groups', str(id)])
    return handle_request('get', url, wrapper=SharedIPGroup,
                          container='sharedIpGroup')


def create(name, server):
    """Create a Shared IP Group.

    :param name: Name of the Shared IP Group
    :type name: str
    :param server: The Server or ``id`` to add to group
    :type server: int or :class:`vaporize.servers.Server`
    :returns: A shiny new CloudServers Shared IP Group.
    :rtype: :class:`SharedIPGroup`

    .. versionadded:: 0.1
    """
    if isinstance(server, vaporize.servers.Server):
        server = server.id
    server = int(server)
    data = {'sharedIpGroup': {'name': name,
                              'server': server}}
    data = json.dumps(data)
    url = '/'.join([get_url('cloudservers'), 'server_ip_groups'])
    return handle_request('post', url, data, SharedIPGroup, 'sharedIpGroup')



# -*- coding: utf-8 -*-

import json

from vaporize.core import convert_datetime, get_url, handle_request, query
from vaporize.utils import DotDict


class Pointer(DotDict):
    """A CloudDNS Reverse DNS Pointer (PTR)."""
    pass






# -*- coding: utf-8 -*-

from vaporize.core import get_url, handle_request, query
from vaporize.utils import DotDict


class Flavor(DotDict):
    """A CloudServers Flavor."""
    def __repr__(self):
        if 'name' in self:
            return '<Flavor %s>' % self['name']
        return super(Flavor, self).__repr__()


def list(limit=None, offset=None, detail=False):
    """Returns a list of Flavors.

    :param limit: Limit the result set by a number
    :type limit: int
    :param offset: Offset the result set by a number
    :type offset: int
    :param detail: Return additional details about each Flavor
    :type: bool
    :returns: A list of CloudServers Flavors.
    :rtype: :class:`Flavor`

    .. versionadded:: 0.1
    """
    url = [get_url('cloudservers'), 'flavors']
    if detail:
        url.append('detail')
    url = '/'.join(url)
    if limit is not None or offset is not None:
        url = query(url, limit=limit, offset=offset)
    return handle_request('get', url, wrapper=Flavor, container='flavors')


def get(id):
    """Returns a Flavor by ID.

    :param id: The ID of the Flavor to retrieve
    :type id: int
    :returns: A CloudServers Flavor matching the ID.
    :rtype: :class:`Flavor`

    .. versionadded:: 0.1
    """
    url = '/'.join([get_url('cloudservers'), 'flavors', str(id)])
    return handle_request('get', url, wrapper=Flavor, container='flavor')



# -*- coding: utf-8 -*-

import json
import time
try:
    # Python 3.x
    from urllib.parse import parse_qsl, urlencode, urlsplit, urlunsplit
except ImportError:
    # Python 2.x
    from urllib import urlencode
    from urlparse import parse_qsl, urlsplit, urlunsplit

import dateutil.parser
import requests

from vaporize.exceptions import ConnectionError, handle_exception

US_AUTH_URL = "https://identity.api.rackspacecloud.com/v2.0/tokens"
UK_AUTH_URL = "https://lon.identity.api.rackspacecloud.com/v2.0/tokens"

_settings = {}
_session = {}


class Auth(requests.auth.AuthBase):
    def __init__(self, token):
        self.token = token

    def __call__(self, r):
        r.headers['X-Auth-Token'] = self.token
        return r


def connect(user, apikey, region='DFW'):
    """Create a session with the Rackspace Cloud API.

    .. note::

        Region support is not universal across all Rackspace Cloud services.

    :param user: A Rackspace Cloud username.
    :type user: str
    :param apikey: A Rackspace Cloud API key.
    :type apikey: str
    :param region: A Rackspace Cloud region, such as ``DFW``, ``ORD`` or ``LON``.
    :type region: str

    Raises::
        ConnectionError: An error occured while creating the connection/session.
    """
    global _settings, _session
    if region in ['DFW', 'ORD']:
        auth_url = US_AUTH_URL
    elif region == 'LON':
        auth_url = UK_AUTH_URL
    else:
        raise ConnectionError("Unrecognized region %s" % region)
    headers = {'Content-Type': 'application/json',
               'Accept': 'application/json'}
    data = json.dumps({'auth': {
        'RAX-KSKEY:apiKeyCredentials': {
            'username': user,
            'apiKey': apikey
            }
        }})
    response = requests.post(auth_url, headers=headers, data=data)
    if response.status_code in [200, 203]:
        data = json.loads(response.content)['access']
        _settings['token'] = data['token']['id']
        _settings['expires'] = dateutil.parser.parse(data['token']['expires'])
        for service in data['serviceCatalog']:
            name = service['name'].lower()
            if len(service['endpoints']) == 1:
                url = service['endpoints'][0]['publicURL']
            else:
                for endpoint in service['endpoints']:
                    if endpoint['region'] == region:
                        url = endpoint['publicURL']
            _settings[name + '_url'] = url
        auth = Auth(_settings['token'])
        headers = {'Content-Type': 'application/json',
                   'Accept': 'application/json'}
        _session = requests.session(auth=auth, headers=headers)
    else:
        raise ConnectionError("HTTP %d: %s" % (response.status_code,
                                               response.content))


def handle_request(verb, url, data=None, wrapper=None, container=None, **kwargs):
    """Handle a request/response in a consistent way."""
    session = get_session()
    if verb == 'get':
        request = session.get
        url = munge_url(url)
    elif verb == 'post':
        request = session.post
    elif verb == 'put':
        request = session.put
    elif verb == 'delete':
        request = session.delete
    response = request(url, data=data)
    if response.status_code not in [200, 201, 202, 203, 204]:
        handle_exception(response.status_code, response.content)
    content = response.content.strip()
    if not content:
        return True
    content = json.loads(content)
    if container and isinstance(content[container], list):
        return [wrapper(i, **kwargs) for i in content[container]]
    elif container is None:
        return wrapper(content, **kwargs)
    else:
        return wrapper(content[container], **kwargs)


def get_session():
    """Return the requests session."""
    return _session


def get_url(service):
    """Return the URL for the specific Rackspace Cloud service."""
    service = '%s_url' % service
    if service in _settings:
        return _settings[service]
    else:
        raise ConnectionError('Not connected to Rackspace Cloud')


def query(url, **kwargs):
    """Append to the URL's query string."""
    scheme, netloc, path, query, fragment = urlsplit(url)
    query = parse_qsl(query)
    for k, v in list(kwargs.items()):
        if v is not None:
            query.append((k, v))
    query = urlencode(query)
    return urlunsplit((scheme, netloc, path, query, fragment))


def munge_url(url):
    """Prevent GET responses from being aggressively cached."""
    scheme, netloc, path, query, fragment = urlsplit(url)
    query = parse_qsl(query)
    query.append(('fresh', str(time.time())))
    query = urlencode(query)
    return urlunsplit((scheme, netloc, path, query, fragment))


def convert_datetime(value):
    return dateutil.parser.parse(value)



# -*- coding: utf-8 -*-

import json

from vaporize.core import convert_datetime, get_url, handle_request, query
import vaporize.servers
from vaporize.utils import DotDict


class Image(DotDict):
    """A CloudServers Image."""
    def __repr__(self):
        if 'name' in self:
            return '<Image %s>' % self['name']
        return super(Image, self).__repr__()

    def __setitem__(self, key, value):
        if key == 'serverId':
            key = 'server_id'
        elif key in ['created', 'updated']:
            value = convert_datetime(value)
        super(Image, self).__setitem__(key, value)

    def delete(self):
        """Delete this Image.

        .. note::

            You can only delete Images you create.

        .. warning::

            There is not confirmation step for this operation. Deleting an
            image is permanent.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudservers'), 'images', str(self['id'])])
        handle_request('delete', url)


def list(limit=None, offset=None, detail=False):
    """Returns a list of CloudServers Images.

    :param limit: Limit the result set by a cetain number
    :type limit: int
    :param offset: Offset the result set by a certain number
    :type offset: int
    :param detail: Return additional details about each Image
    :type detail: bool
    :returns: A list of CloudServers Images.
    :rtype: A list of :class:`Image`

    .. versionadded:: 0.1
    """
    url = [get_url('cloudservers'), 'images']
    if detail:
        url.append('detail')
    url = '/'.join(url)
    if limit is not None or offset is not None:
        url = query(url, limit=limit, offset=offset)
    return handle_request('get', url, wrapper=Image, container='images')


def get(id):
    """Return an Image by ID.

    :param id: The ID of the Image to retrieve
    :type id: int
    :returns: A CloudServer Image matching the ID.
    :rtype: :class:`Image`

    .. versionadded:: 0.1
    """
    url = '/'.join([get_url('cloudservers'), 'images', str(id)])
    return handle_request('get', url, wrapper=Image, container='image')


def create(name, server):
    """Create an Image.

    :param name: Name of the Image
    :type name: str
    :param server: Server or ``id`` to base the Image upon
    :type server: int or :class:`vaporize.servers.Server`
    :returns: A shiny new CloudServers Image.
    :rtype: :class:`Image`

    .. versionadded:: 0.1
    """
    if isinstance(server, vaporize.servers.Server):
        server = server.id
    server = int(server)
    data = {'image': {'serverId': server,
                      'name': name}}
    data = json.dumps(data)
    url = '/'.join([get_url('cloudservers'), 'images'])
    return handle_request('post', url, data, Image, 'image')



# -*- coding: utf-8 -*-

import json

from vaporize.core import convert_datetime, get_url, handle_request, query
from vaporize.utils import DotDict


class AccessRule(DotDict):
    """A CloudLoadBalancer Access List Rule."""
    def __repr__(self):
        if 'type' in self and 'address' in self:
            return '<AccessRule %s %s>' % (self['type'], self['address'])
        return super(AccessRule, self).__repr__()

    @classmethod
    def create(cls, type, address):
        """Create an Access Rule.

        :param type: ``ACCEPT`` or ``DENY``
        :type type: str
        :param address: The IP address in which to apply the rule.
        :type address: str
        :returns: A shiny new Access Rule.
        :rtype: :class:`AccessRule`

        .. versionadded:: 0.1
        """
        return cls(type=type, address=address)

    def delete(self):
        """Delete this Access Rule.

        .. warning::

            There is no confirmation step for this operation.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        assert 'loadbalancer_id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['loadbalancer_id']), 'accesslist',
                        str(self['id'])])
        handle_request('delete', url)


class Algorithm(DotDict):
    """A CloudLoadBalancer Algorithm."""
    def __repr__(self):
        if 'name' in self:
            return '<Algorithm %s>' % self['name']
        return super(Algorithm, self).__repr__()


class AllowedDomain(DotDict):
    """A CloudLoadBalancer Allowed Domains."""
    def __setitem__(self, key, value):
        if key == 'allowedDomain':
            key = 'name'
            value = value['name']
        super(AllowedDomain, self).__setitem__(key, value)


class ContentCaching(DotDict):
    """A CloudLoadBalancer Content Caching."""
    pass


class ConnectionLogging(DotDict):
    """A CloudLoadBalancer Connection Logging."""
    pass


class ConnectionThrottle(DotDict):
    """A CloudLoadBalancer Connection Throttle."""
    pass


class ErrorPage(DotDict):
    """A CloudLoadBalancer Custom Error Page."""
    pass


class HealthMonitor(DotDict):
    """A CloudLoadBalancer Health Monitor."""
    pass


class LoadBalancer(DotDict):
    """A CloudLoadBalancer Load Balancer."""
    def __repr__(self):
        if 'name' in self:
            return '<LoadBalancer %s>' % self['name']
        return super(LoadBalancer, self).__repr__()

    def __setitem__(self, key, value):
        if key == 'accessList':
            value = [AccessRule(v) for v in value]
        elif key == 'connectionLogging':
            value = ConnectionLogging(value)
        elif key == 'connectionThrottle':
            value = ConnectionThrottle(value)
        elif key == 'contentCaching':
            value = ContentCaching(value)
        elif key == 'errorpage':
            value = ErrorPage(value)
        elif key == 'healthMonitor':
            value = HealthMonitor(value)
        elif key == 'nodes':
            value = [Node(v) for v in value]
        elif key == 'sessionPersistence':
            value = SessionPersistence(value)
        elif key == 'virtualIps':
            value = [VirtualIP(v) for v in value]
        elif key in ['created', 'updated']:
            value = convert_datetime(value['time'])
        super(LoadBalancer, self).__setitem__(key, value)

    def reload(self):
        """Reload this Load Balancer (an implicit :func:`get`).

        .. versionadded:: 0.1
        """
        assert 'id' in self
        response = get(self['id'])
        self.update(response)
        return self

    def modify(self, name=None, protocol=None, port=None, algorithm=None,
               connection_logging=False):
        """Modify this Load Balancer's properties.

        :param name: The Load Balancer's name.
        :type name: str
        :param protocol: A Load Balancer protocol, see: :func:`protocols`.
        :type protocol: str
        :param port: A Load Balancer port, see :func:`protocols`.
        :type port: int
        :param algorithm: A Load Balancer Algorithm, see :func:`algorithms`.
        :type algorithm: str
        :param connection_logging: Enable or disable Connection Logging.
        :type connection_logging: bool
        :returns: An updated Load Balancer.
        :rtype: :class:`LoadBalancer`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = {'loadBalancer': {}}
        if name is not None:
            data['loadBalancer']['name'] = name
        if algorithm is not None:
            data['loadBalancer']['algorithm'] = algorithm
        if protocol is not None:
            data['loadBalancer']['protocol'] = protocol
        if port is not None:
            data['loadBalancer']['port'] = int(port)
        if connection_logging is not None:
            connection_logging = bool(connection_logging)
            data['loadBalancer']['connectionLogging'] = connection_logging
        data = json.dumps(data)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id'])])
        handle_request('put', url, data)
        if name is not None:
            self['name'] = name
        if algorithm is not None:
            self['algorithm'] = algorithm
        if protocol is not None:
            self['protocol'] = protocol
        if port is not None:
            self['port'] = int(port)
        if connection_logging is not None:
            self['connectionLogging']['enabled'] = bool(connection_logging)
        return self

    def delete(self):
        """Delete this Load Balancer.

        .. warning::

            There is not confirmation step for this operation. Deleting a Load
            Balancer is permanent.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id'])])
        handle_request('delete', url)

    def nodes(self):
        """Returns a list of Nodes for this Load Balancer.

        :returns: A list of Nodes.
        :rtype: list of :class:`Node`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'nodes'])
        self['nodes'] = handle_request('get', url, wrapper=Node,
                                       container='nodes',
                                       loadbalancer_id=self['id'])
        return self['nodes']

    def add_nodes(self, *nodes):
        """Add Nodes to this Load Balancer.

            >>> loadbalancer = vaporize.loadbalancers.create(...)
            >>> node1 = vaporize.loadbalancers.Node.create(....)
            >>> node2 = vaporize.loadbalancers.Node.crete(....)
            >>> loadbalancer.add_nodes(node1, node2)

        :param nodes: Nodes to add to the Load Balancer.
        :type nodes: :class:`Node`
        :returns: A list of Nodes.
        :rtype: list of :class:`Node`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = {'nodes': []}
        for node in nodes:
            if isinstance(node, Node):
                data['nodes'].append({
                    'address': node.address,
                    'port': node.port,
                    'condition': node.condition,
                    'type': node.type,
                    'weight': node.weight
                    })
        data = json.dumps(data)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'nodes'])
        self['nodes'] = handle_request('post', url, data, Node, 'nodes',
                                       loadbalancer_id=self['id'])
        return self['nodes']

    def remove_node(self, node):
        """Remove a Node from this Load Balancer.

        :param node: ``id`` or :class:`Node` to remove from Load Balancer
        :type node: int or :class:`Node`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        node = node.id if isinstance(node, Node) else int(node)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'nodes', str(node)])
        handle_request('delete', url)

    def virtual_ips(self):
        """Returns a list of VirtualIPs for this Load Balancer.

        :returns: A list of Virtual IPs on this Load Balancer.
        :rtype: list of :class:`VirtualIP`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'virtualips'])
        response = handle_request('get', url, wrapper=VirtualIP,
                                  container='virtualIps',
                                  loadbalancer_id=self['id'])
        self['virtual_ips'] = response
        return self['virtual_ips']

    def add_virtual_ips(self, *virtual_ips):
        """Add Virtual IPs to this Load Balancer.

            >>> loadbalancer = vaporize.loadbalancers.create(...)
            >>> virtual_ip1 = vaporize.loadbalancers.VirtualIP.create(...)
            >>> virtual_ip2 = vaporize.loadbalancers.VirtualIP.create(...)
            >>> loadbalancer.add_virtual_ips(virtual_ip1, virtual_ip2)

        :param virtual_ips: Virtual IPs to add to thisLoad Balancer
        :type virtual_ips: :class:`VirtualIP`
        :returns: A list of Virtual IPs on the Load Balancer
        :rtype: list of :class:`VirtualIP`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = {'virtualIps': []}
        for virtual_ip in virtual_ips:
            if isinstance(virtual_ip, VirtualIP):
                data['virtualIps'].append({
                    'ipVersion': virtual_ip.version,
                    'type': virtual_ip.type
                    })
        data = json.dumps(data)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'virtualips'])
        self['virtual_ips'] = handle_request('post', url, data, VirtualIP,
                                             'virtualIps',
                                             loadbalancer_id=self['id'])
        return self['virtual_ips']

    def remove_virtual_ip(self, virtual_ip):
        """Remove a VirtualIP from this Load Balancer.

        :param virtual_ip: ``id`` or Virtual IP to remove from Load Balancer
        :type virtual_ip: int or :class:`VirtualIP`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        if isinstance(virtual_ip, VirtualIP):
            virtual_ip = virtual_ip.id
        virtual_ip = int(virtual_ip)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'virtualips', str(virtual_ip)])
        handle_request('delete', url)

    def access_list(self):
        """Returns a list of AccessRules for this Load Balancer.

        :returns: A list of Access List Rules for this Load Balancer.
        :rtype: list of :class:`AccessRule`.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'accesslist'])
        self['access_list'] = handle_request('get', url,
                                             wrapper=AccessRule,
                                             container='accessList',
                                             loadbalancer_id=self['id'])
        return self['access_list']

    def add_access_rules(self, *access_rules):
        """Add AccessRules to this Load Balancer.

            >>> loadbalancer = vaporize.loadbalancers.create(...)
            >>> access_rule1 = vaporize.loadbalancers.AccessRule.create(...)
            >>> access_rule2 = vaporize.loadbalancers.AccessRule.create(....)
            >>> loadbalancer.add_access_rules(access_rule1, access_rule2)

        :param access_rules: Access Rules to add to this Load Balancer.
        :type access_rules: :class:`AccessRule`
        :returns: A list of Access Rules.
        :rtype: list of :class:`AccessRule`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = {'accessList': []}
        for access_rule in access_rules:
            if isinstance(access_rule, AccessRule):
                data['accessList'].append({
                    'type': access_rule.type,
                    'address': access_rule.address
                    })
        data = json.dumps(data)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'accesslist'])
        self['access_list'] = handle_request('post', url, data, AccessRule,
                                             'accessList',
                                             loadbalancer_id=self['id'])
        return self['access_list']

    def remove_access_rule(self, access_rule):
        """Remove an AccessRule from this Load Balancer.

        :param access_rule: ``id`` or Access Rule to remove from Load Balancer
        :type access_rule: int or :class:`AccessRule`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        if isinstance(access_rule, AccessRule):
            access_rule = access_rule.id
        access_rule = int(access_rule)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'accesslist', str(access_rule)])
        handle_request('delete', url)

    def connection_logging(self):
        """Returns the ConnectionLogging setting for this Load Balancer.

        :returns: This Load Balancer's Content Logging setting.
        :rtype: :class:`ConnectionLogging`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'connectionlogging'])
        response = handle_request('get', url, wrapper=ConnectionLogging,
                                  container='connectionLogging')
        self['connection_logging'] = response
        return self['connection_logging']

    def enable_connection_logging(self):
        """Enable Connection Logging for this Load Balancer.

        :returns: This Load Balancer's Connection Logging setting.
        :rtype: :class:`ConnectionLogging`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'connectionLogging': {'enabled': True}})
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'connectionlogging'])
        handle_request('put', url, data)
        self['connection_logging']['enabled'] = True

    def disable_connection_logging(self):
        """Disable Connection Logging for this Load Balancer.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'connectionLogging': {'enabled': False}})
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'connectionlogging'])
        handle_request('put', url, data)
        self['connection_logging']['enabled'] = False

    def content_caching(self):
        """Returns the Connection Caching setting for this Load Balancer.

        :returns: This Load Balancer's Content Caching setting.
        :rtype: :class:`ContentCaching`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'contentcaching'])
        self['content_caching'] = handle_request('get', url,
                                                 wrapper=ContentCaching,
                                                 container='contentCaching')
        return self['content_caching']

    def enable_content_caching(self):
        """Enable Content Caching for this Load Balancer.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'contentCaching': {'enabled': True}})
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'contentCaching'])
        handle_request('put', url, data)
        self['content_caching']['enabled'] = True

    def disable_content_caching(self):
        """Disable Content Caching for this Load Balancer.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'contentCaching': {'enabled': False}})
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'contentcaching'])
        handle_request('put', url, data)
        self['content_caching']['enabled'] = False

    def connection_throttle(self):
        """Return the Connection Throttle setting for this Load Balancer.

        :returns: This Load Balancer's Connection Throttle setting.
        :rtype: :class:`ConnectionThrottle`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'connectionthrottle'])
        response = handle_request('get', url, wrapper=ConnectionThrottle,
                                  container='connectionThrottle')
        self['connection_throttle'] = response
        return self['connection_throttle']

    def enable_connection_throttle(self, max_connections, min_connections,
                                   max_connection_rate, rate_interval):
        """Enable Connection Throttle setting for this Load Balancer.

        :param max_connections: Max connections.
        :type max_connections: int
        :param min_connections: Minimum connections required to throttle.
        :type min_connections: int
        :param max_connection_rate: Maximum rate of connections.
        :type max_connection_rate: int
        :param rate_interval: Rate interval in seconds.
        :type rate_interval: int
        :returns: This Load Balancer's Connection Throttle
        :rtype: :class:`ConnectionThrottle`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = {'connectionThrottle': {
            'maxConnections': int(max_connections),
            'minConnections': int(min_connections),
            'maxConnectionRate': int(max_connection_rate),
            'rateInterval': int(rate_interval)
            }}
        data = json.dumps(data)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'connectionthrottle'])
        handle_request('put', url, data)
        self['connection_throttle'].update({
                'max_connections': int(max_connections),
                'min_connections': int(min_connections),
                'max_connection_rate': int(max_connection_rate),
                'rate_interval': int(rate_interval)
                })
        return self['connection_throttle']

    def disable_connection_throttle(self):
        """Disable Connection Throttle for this Load Balancer.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'connectionthrottle'])
        handle_request('delete', url)
        self['connection_throttle'] = {}

    def health_monitor(self):
        """Returns the Health Monitor setting for this Load Balancer.

        :returns: This Load Balancer's Health Monitor setting.
        :rtype: :class:`HealthMonitor`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'healthmonitor'])
        return handle_request('get', url, wrapper=HealthMonitor,
                              container='healthMonitor')

    def enable_health_monitor(self, type, delay, timeout,
                              attempts_before_deactivation):
        """Enable Health Monitor for this Load Balancer.

        :returns: A Health Monitor setting.
        :rtype: :class:`HealthMonitor`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = {'healthMonitor': {
            'type': type,
            'delay': int(delay),
            'timeout': int(timeout),
            'attemptsBeforeDeactivation': int(attempts_before_deactivation)
            }}
        data = json.dumps(data)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'healthmonitor'])
        return handle_request('put', url, data, HealthMonitor, 'healthMonitor')

    def disable_health_monitor(self):
        """Disable Health Monitor for this Load Balancer.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'healthmonitor'])
        handle_request('delete', url)

    def session_persistence(self):
        """Return Session Persistence setting for this Load Balancer.

        :returns: This Load Balancer's Session Persistence setting.
        :rtype: :class:`SessionPersistence`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'sessionpersistence'])
        return handle_request('get', url, wrapper=SessionPersistence,
                              container='sessionPersistence')

    def enable_session_persistence(self, type):
        """Enable Session Persistence for this Load Balancer.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'sessionPersistence': {'persistenceType': type}})
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'sessionpersistence'])
        handle_request('put', url, data)

    def disable_session_persistence(self):
        """Disable Session Persistance for this Load Balancer.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'sessionpersistence'])
        handle_request('delete', url)

    def error_page(self):
        """Returns the Error Page for this Load Balancer.

        :returns: This Load Balancer's Error Page setting.
        :rtype: :class:`ErrorPage`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'errorpage'])
        return handle_request('get', url, wrapper=ErrorPage,
                              container='errorpage')

    def set_error_page(self, content):
        """Set a Custom Error Page for this Load Balancer.

        :param content: Contents of the custom error page.
        :type content: str

        .. versionadded:: 0.1
        """
        assert 'id' in self
        data = json.dumps({'errorpage': {'content': content}})
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'errorpage'])
        handle_request('put', url, data)

    def reset_error_page(self):
        """Reset the Error Page for this Load Balancer.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'errorpage'])
        handle_request('delete', url)

    def stats(self):
        """Returns stats for this Load Balancer.

        :returns: Stats for this Load Balancer.
        :rtype: :class:`Stat`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['id']), 'stats'])
        return handle_request('get', url, wrapper=Stat)

    def usage(self, start_time=None, end_time=None):
        """Returns Usage Report for this Load Balancer.

        :param start_time: A datetime string as a starting point.
        :type start_time: str or datetime
        :param end_time: A datetime string as an ending point.
        :type end_time: str or datetime
        :returns: A usage report for this Load Balancer.
        :rtype: :class:`UsageReport`

        .. versionadded:: 0.1
        """
        url = [get_url('cloudloadbalancers'), 'loadbalancers',
               str(self['id']), 'usage']
        if start_time is not None and end_time is not None:
            url = '/'.join(url)
            url = query(url, startTime=str(start_time), endTime=str(end_time))
        else:
            url.append('current')
            url = '/'.join(url)
        return handle_request('get', url, wrapper=UsageReport,
                              container='loadBalancerUsage')


class Node(DotDict):
    """A CloudLoadBalancer Node."""
    def __repr__(self):
        if 'address' in self:
            return '<Node %s>' % self['address']
        return super(Node, self).__repr__()

    @classmethod
    def create(cls, address, port, condition, type, weight):
        """Create a Load Balancer Node.

        .. versionadded:: 0.1
        """
        return cls(address=address, port=int(port), condition=condition,
                   type=type, weight=int(weight))

    def reload(self):
        """Reload this Node.

        :returns: An pudated Node.
        :rtype: :class:`Node`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        assert 'loadbalancer_id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['loadbalancer_id']), 'nodes',
                        str(self['id'])])
        response = handle_request('get', url, wrapper=Node, container='node',
                                  loadbalancer_id=self['loadbalancer_id'])
        self.update(response)
        return self

    def modify(self, condition=None, type=None, weight=None):
        """Modify a Node's properties.

        :param condition: A Node condition; ``ENABLED``, ``DISABLED`` or
            ``DRAINING``.
        :type condition: str
        :param type: A Node type.
        :type type: str
        :param weight: A Node weight.
        :type weight: int
        :returns: An updated Node.
        :rtype: :class:`Node`

        .. versionadded:: 0.1
        """
        assert 'id' in self
        assert 'loadbalancer_id' in self
        data = {'node': {}}
        if condition is not None and condition in ['ENABLED', 'DISABLED',
                'DRAINING']:
            data['node']['condition'] = condition
        if type is not None and type in ['PRIMARY', 'SECONDARY']:
            data['node']['type'] = type
        if weight is not None:
            data['node']['weight'] = int(weight)
        data = json.dumps(data)
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['loadbalancer_id']), 'nodes',
                        str(self['id'])])
        response = handle_request('put', url, data)
        if response:
            if condition is not None and condition in ['ENABLED', 'DISABLED',
                    'DRAINING']:
                self['condition'] = condition
            if type is not None and type in ['PRIMARY', 'SECONDARY']:
                self['type'] = type
            if weight is None:
                self['weight'] = int(weight)
        return self

    def delete(self):
        """Delete this Node.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        assert 'loadbalancer_id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['loadbalancer_id']), 'nodes',
                        str(self['id'])])
        handle_request('delete', url)


class Protocol(DotDict):
    """A CloudLoadBalancer Protocol."""
    def __repr__(self):
        if 'name' in self:
            return '<Protocol %s>' % self['name']
        return super(Protocol, self).__repr__()


class SessionPersistence(DotDict):
    """A CloudLoadBalancer Session Persistence."""
    def __repr__(self):
        if 'persistenceType' in self:
            return '<SessionPersistence %s>' % self['persistenceType']
        return super(SessionPersistence, self).__repr__()

    def __setitem__(self, key, value):
        if key == 'persistenceType':
            key = 'type'
        super(SessionPersistence, self).__setitem__(key, value)


class UsageReport(DotDict):
    """A CloudLoadBalancer Usage Report."""
    pass


class Stat(DotDict):
    pass


class VirtualIP(DotDict):
    """A CloudLoadBalancer Virtual IP."""
    def __repr__(self):
        if 'address' in self:
            return '<VirtualIP %s>' % self['address']
        return super(VirtualIP, self).__repr__()

    def __setitem__(self, key, value):
        if key == 'ipVersion':
            key = 'version'
        super(VirtualIP, self).__setitem__(key, value)

    @classmethod
    def create(cls, version, type):
        """Create a Virtual IP.

        :param version: ``IPV4`` or ``IPV6``.
        :type version: str
        :param type: ``PUBLIC`` or ``SERVICENET``
        :type type: str
        :returns: A shiny new Virtual IP.
        :rtype: :class:`VirtualIP`

        .. versionadded:: 0.1
        """
        return cls(version=version, type=type)

    def delete(self):
        """Delete this Virtual IP.

        .. versionadded:: 0.1
        """
        assert 'id' in self
        assert 'loadbalancer_id' in self
        url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers',
                        str(self['loadbalancer_id']), 'virtualips',
                        str(self['id'])])
        handle_request('delete', url)


def list(limit=None, offset=None, marker=None, node=None, deleted=False):
    """Returns a list of Load Balancers.

    :returns: A list of CloudLoadBalancer Load Balancers.
    :rtype: A list of :class:`LoadBalancer`.

    .. versionadded:: 0.1
    """
    url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers'])
    if limit is not None or offset is not None or marker is not None:
        url = query(url, limit=limit, offset=offset, marker=marker)
    if deleted:
        url = query(url, status='DELETED')
    if node is not None:
        url = query(url, nodeaddress=node)
    return handle_request('get', url, wrapper=LoadBalancer,
                          container='loadBalancers')


def get(id):
    """Return a Load Balancer by ID.

    :param id: A Load Balancer ID.
    :type id: int
    :returns: A CloudLoadBalancer Load Balancer with this ID.
    :rtype: :class:`LoadBalancer`

    .. versionadded:: 0.1
    """
    url = '/'.join([get_url('cloudloadbalancers'), 'flavors', str(id)])
    return handle_request('get', url, wrapper=LoadBalancer,
                          container='loadBalancer')


def create(name, protocol, port, virtual_ips, nodes, algorithm=None,
           access_list=None, connection_logging=None, connection_throttle=None,
           health_monitor=None, session_persistence=None, metadata=None):
    """Create a Load Balancer.

    :returns: A shiny new CloudLoadBalancer Load Balancer.
    :rtype: :class:`LoadBalancer`

    .. versionadded:: 0.1
    """
    data = {'loadBalancer': {'name': name,
                             'protocol': protocol,
                             'port': int(port),
                             'virtualIps': [],
                             'nodes': [],
                             'metadata': metadata or {}}}
    for virtual_ip in virtual_ips:
        data['loadBalancer']['virtualIps'].append({
            'ipVersion': virtual_ip.version,
            'type': virtual_ip.type
            })
    for node in nodes:
        data['loadBalancer']['nodes'].append({'address': node.address,
                                              'port': int(node.port),
                                              'condition': node.condition})
    if connection_logging is not None:
        data['loadBalancer']['connectionLogging'] = {
                'enabled': bool(connection_logging)
                }
    data = json.dumps(data)
    url = '/'.join([get_url('cloudloadbalancers'), 'loadbalancers'])
    return handle_request('post', url, data, LoadBalancer, 'loadBalancer')


def algorithms(limit=None, offset=None, marker=None):
    """Return a list of supported algorithms.

    :param limit: Limit the result set by a certain amount.
    :type limit: int
    :param offset: Offset the result set by a certain amount.
    :type offset: int
    :param marker: Start result set at a specific marker (ID).
    :type marker: int
    :returns: A list of supported CloudLoadBalancer algorithms.
    :rtype: A list of :class:`Algorithm`.

    .. versionadded:: 0.1
    """
    url = [get_url('cloudloadbalancers'), 'loadbalancers', 'algorithms']
    url = '/'.join(url)
    if limit is not None or offset is not None or marker is not None:
        url = query(url, limit=limit, offset=offset, marker=marker)
    return handle_request('get', url, wrapper=Algorithm,
                          container='algorithms')


def allowed_domains(limit=None, offset=None, marker=None):
    """Return a list of allowed domains.

    :param limit: Limit the result set by a certain amount.
    :type limit: int
    :param offset: Offset the result set by a certain amount.
    :type offset: int
    :param marker: Start result set at a specific marker (ID).
    :type marker: int
    :returns: A list of CloudLoadBalancer allowed domains.
    :rtype: A list of :class:`AllowedDomain`.

    .. versionadded:: 0.1
    """
    url = [get_url('cloudloadbalancers'), 'loadbalancers', 'alloweddomains']
    url = '/'.join(url)
    if limit is not None or offset is not None or marker is not None:
        url = query(url, limit=limit, offset=offset, marker=marker)
    return handle_request('get', url, wrapper=AllowedDomain,
                          container='allowedDomains')


def protocols(limit=None, offset=None, marker=None):
    """Return a list of supported protocols.

    :param limit: Limit the result set by a certain amount.
    :type limit: int
    :param offset: Offset the result set by a certain amount.
    :type offset: int
    :param marker: Start result set at a specific marker (ID).
    :type marker: int
    :returns: A list of supported CloudLoadBalancer protocols.
    :rtype: A list of :class:`Protocol`.

    .. versionadded:: 0.1
    """
    url = [get_url('cloudloadbalancers'), 'loadbalancers', 'protocols']
    url = '/'.join(url)
    if limit is not None or offset is not None or marker is not None:
        url = query(url, limit=limit, offset=offset, marker=marker)
    return handle_request('get', url, wrapper=Protocol, container='protocols')



#!/usr/bin/env python

import vaporize

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(
    name=vaporize.__title__,
    description='A clean and consistent library for the Rackspace Cloud / OpenStack',
    long_description=open('README.rst').read(),
    author=vaporize.__author__,
    author_email=vaporize.__email__,
    url='https://github.com/kolanos/vaporize',
    version=vaporize.__version__,
    license=vaporize.__license__,
    install_requires=open('requirements.txt').readlines(),
    packages=['vaporize'],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Distributed Computing',
        'Topic :: Utilities',
    ],
)
